/**
 * Enhances node path alias entry with automatic preview on forms.
 */

(function ($) {

  /**
   * Adds behaviors to load on document.ready().
   */
  Drupal.behaviors.alias_preview = {

    attach: function (context, settings) {
      // On node edit forms only, attaches a static "Link URL" preview.
      // Finds any existing alias value,
      // then injects it into the title field description.
      // @TODO: move into a form alter?
      if ($('#edit-path-alias').length) {
        var existing_alias = $('#edit-path-alias').val();
        if (existing_alias.length) {
          var existing_preview = '';
          if (settings.alias_preview.prefix.length) {
            existing_preview += settings.alias_preview.prefix + '/'
          }
          existing_preview += existing_alias;
          var markup = '<strong>URL:</strong> ' + existing_preview;
          markup += ' <a id="pathauto-extra-edit-path" href="#path[pathauto]">edit</a>';
          insert_description_element(markup);
          attach_scroll_behavior();
        }
      }

      // On node add forms, use AJAX callback to generate alias preview.
      $('.page-node-add #edit-title', context).once('aliasPreview', function() {
        $(this).change(function () {
          insert_description_element();
          alias_preview_ajax_handler();
        });
      });

      /**
       * Provides a smooth animation when the user clicks "edit".
       * @TODO: Move scroll behavior to separate js included on all node forms.
       */
      function attach_scroll_behavior() {
        // Brings the user to the custom alias field with proper settings checked.
        $('.form-item-title .alias-preview a').click(function () {
          $('html, body', context).animate({ scrollTop: $('.vertical-tabs').offset().top });
          $('.vertical-tab-button a:contains("URL path settings")', context).click();
          $('#edit-path-pathauto').click().attr('checked', false).attr('value', 0);
          $('.form-item-path-alias').removeClass('form-disabled');
          $('#edit-path-alias').removeAttr("disabled").focus().select();
        });
      }

      /**
       * Adds the AJAX handler to generate pathauto aliases for preview.
       */
      function alias_preview_ajax_handler() {
        // Verifies settings before continuing...
        if (!settings.alias_preview.active) {
          return;
        }

        // Prepares the URL parameters to call for this node.
        var data = settings.alias_preview;
        // Cleans the title value stored in our URL params.
        data.title = $.trim($('#edit-title').attr('value'));
        // Only proceeds if there is a title value, and
        // if the "Generate automatic alias" setting is checked.
        if (!data.title.length || !$('#edit-path-pathauto').attr('checked')) {
          return;
        }

        // Adds autocomplete-style default Drupal throbber.
        $('.form-item-title .description').html('<strong>Link URL:</strong> <span class="ajax-progress"><span class="throbber"></span></span>');
        // Prepares the ajax callback URL to query.
        var href = href = 'http://' + document.location.host + '/node/alias-preview';
        // Fetches the generated alias from the menu callback.
        $.getJSON(href, data, function (json) {
          // If we got a successful AJAX response...
          if (json.status) {

            // Prepares the alias, removing purl.
            var alias = json.data;
            var base_url = settings.alias_preview.prefix;
            var purl = settings.alias_preview.purl;
            if (purl.length) {
              alias = alias.slice(purl.length + 1);
            }

            // Updates the existing pathauto alias field with the new value.
            $('#edit-path-alias').attr('value', alias);
            // Updates the title input field description to immediately show user.
            // @TODO: Unify functions to remove this in favor of existing func.
            // @see insert_description_element()
            var description = '<strong>Link URL:</strong> ' + base_url + '/' + alias + ' <a id="pathauto-extra-edit-path" href="#path[pathauto]">edit</a>';
            $('.form-item-title .description').html(description);
            // @TODO: Move scroll behavior to separate js included on all
            // node forms.
            attach_scroll_behavior();
          }
        });
      }

      /**
       * Ensures that the description <div> exists below the title.
       *
       * It won't exist at first when the page loads.
       */
      function insert_description_element(markup) {
        if (! $('.form-item-title .description').length) {
          // @TODO: Replace $.insertAfter() with form alter default-hidden elem.
          $('<div class="alias-preview description"></div>').insertAfter('#edit-title');
        }
        if (markup.length) {
          $('.form-item-title .description').html(markup);
        }
      }
    }

  };

}(jQuery));